import os
import sys
import time
import warnings
from collections import Counter

import numpy as np
import pandas as pd
from sklearn.model_selection import StratifiedKFold

# from dl.deep6ma_backup_new import TA_keras
from dl.dl_model import TA_keras


warnings.filterwarnings("ignore")
from tensorflow.core.protobuf.config_pb2 import ConfigProto
from tensorflow.python.client.session import InteractiveSession
from config.config import Config
from data.data import get_single_data

from tools.tools import get_label, score2, set_global_determinism

import pickle as pkl
os.environ['TF_XLA_FLAGS'] = '--tf_xla_enable_xla_devices'
config1 = ConfigProto()
config1.gpu_options.allow_growth = True
session = InteractiveSession(config=config1)

def dl_cross_validate(X_fake,y_fake,X_train,y_train,X_test,y_test,skf):
    #-----------------------------------------------------------
    base_models_5fold_prediction = np.zeros([X_train.shape[0]])
    base_models_ind_prediction = np.zeros([len(skf),X_test.shape[0]])
    # skf = list(StratifiedKFold(n_splits=n_folds).split(X_train, y_train))
    for i, (train_index, val_index) in enumerate(skf):
        X_train2, y_train2, X_val, y_val = X_train.iloc[train_index, :], y_train[train_index], X_train.iloc[val_index, :], y_train[val_index]
        # ---------------------pretrain------------------------------
        real_neg = X_train2.iloc[y_train2 == 0, :]
        data_train3 = pd.concat([X_fake, real_neg])
        y_train3 = get_label(data_train3)
        # use the fake DNA to pretrain CNN
        config_global = Config()
        config_global.model_name = dataset + str(i) + "_fake"
        config_global.model_save_path = "result0615/"  # "data/global_pretrain_model_group1.model"
        config_global.load_global_pretrain_model = False
        sm = TA_keras(config_global)
        sm.fit(data_train3, y_train3)
        #--------------------------------------------------------------
        config_temp = Config()
        config_temp.model_name = dataset + str(i) + "_tran"
        config_temp.model_save_path = "result0615/"  # "data/global_pretrain_model_group1.model"
        config_temp.load_global_pretrain_model = True
        config_temp.global_model_save_path = config_global.model_save_path + dataset + str(i) + "_fake_.h5"
        ta = TA_keras(config_temp)
        ta.fit(X_train2, y_train2)
        y_pred = ta.predict_proba(X_val)
        base_models_5fold_prediction[val_index] = y_pred
        #-------------------------------------------------
        y_pred_test = ta.predict_proba(X_test)
        base_models_ind_prediction[i,:] = y_pred_test
    result_train = score2(y_train,base_models_5fold_prediction)
    print("5 fold cv:",result_train)
    base_models_ind_prediction_avg = base_models_ind_prediction.mean(axis=0)
    result_test = score2(y_test, base_models_ind_prediction_avg)
    print("ind test:", result_test)
    return result_train,result_test



if __name__ == "__main__":
    sys.setrecursionlimit(15000)
    start = time.time()
    SEED = 0
    set_global_determinism(SEED)

    dataset = "ac4c"

    original_train1, original_test1 = get_single_data(dataset)  # DataFrame with 2 columns (label, seq)
    train_label1 = get_label(original_train1)
    test_label1 = get_label(original_test1)

    # with open("result/fake_data_02.pkl", "rb") as f:
    with open("result/fake_data_bn.pkl", "rb") as f:
        dataset_fake = pkl.load(f)

    n_folds = 5
    result_trains = []
    result_tests = []
    skf = list(StratifiedKFold(n_splits=n_folds).split(original_train1, train_label1))
    for i in range(len(dataset_fake)):
        start_temp = time.time()
        set_global_determinism(SEED)
        y_fake1 = get_label(dataset_fake[i])
        result_train, result_test = dl_cross_validate(dataset_fake[i],y_fake1,original_train1,train_label1,original_test1,test_label1,skf)
        result_trains.append(result_train)
        result_tests.append(result_test)
        end_temp = time.time()
        print("Time consuming:%.2f秒" % (end_temp - start_temp))

    result_trains = pd.DataFrame(result_trains)
    result_tests = pd.DataFrame(result_tests)
    result_trains.to_csv("result0615/result_trains.csv")
    result_tests.to_csv("result0615/result_tests.csv")
    end = time.time()
    print("Time consuming:%.2f秒" % (end - start))

