from colorama import Fore
from lightgbm import LGBMClassifier
from sklearn.decomposition import PCA
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
from sklearn.linear_model import Lasso
from sklearn.metrics import accuracy_score, roc_auc_score
from sklearn.preprocessing import StandardScaler
from tqdm import tqdm
from xgboost import XGBClassifier

from fs.encode1 import ANF, binary, CKSNAP, EIIP, ENAC, Kmer, NAC, PseEIIP, NCP, ENAC2
import numpy as np
import pandas as pd
import random

# from tools.tools import split_h
def sample_h(seqs_df, sample_num_half):
    data_size = seqs_df.shape[0]
    # labels = [(0 if x.find('+') == -1 else 1) for x in seqs_df["seq_id"]]
    labels = seqs_df["label"].values
    labels_index = np.array(range(0, data_size))
    labels_p = [(True if value == 1 else False) for value in labels]
    labels_n = [(True if value == 0 else False) for value in labels]
    labels_p = set(labels_index[labels_p])  # 正样本下标
    labels_n = set(labels_index[labels_n])  # 负样本下标

    indexes_shuffle_p = np.random.choice(list(labels_p), len(labels_p), replace=False)  # 打乱顺序
    indexes_shuffle_n = np.random.choice(list(labels_n), len(labels_n), replace=False)  # 打乱顺序

    indexes_choose_p, indexes_choose_n = list(indexes_shuffle_p[:sample_num_half]), list(
        indexes_shuffle_n[:sample_num_half])
    indexes_remain_p, indexes_remain_n = list(indexes_shuffle_p[sample_num_half:]), list(
        indexes_shuffle_n[sample_num_half:])

    indexes_choose_p.extend(indexes_choose_n)
    indexes_remain_p.extend(indexes_remain_n)

    random.shuffle(indexes_choose_p)
    random.shuffle(indexes_remain_p)
    data_choose = seqs_df.iloc[indexes_choose_p, :]
    data_remain = seqs_df.iloc[indexes_remain_p, :]
    data_choose.index = range(data_choose.shape[0])
    data_remain.index = range(data_remain.shape[0])
    return data_choose,data_remain

def split_h(data, label1=None, freq=0.2):
    data_size = len(data)
    data1 = pd.DataFrame(data)
    if label1 is None:
        labels = np.array(data1.iloc[:, 1])
    else:
        labels = np.array(label1)
    labels_index = np.array(range(0, data_size))
    labels_p = [(True if value == 1 else False) for value in labels]
    labels_n = [(True if value == 0 else False) for value in labels]
    labels_p = set(labels_index[labels_p])
    labels_n = set(labels_index[labels_n])

    labels_p_test = set(random.sample(labels_p, int(len(labels_p) * freq)))
    labels_n_test = set(random.sample(labels_n, int(len(labels_n) * freq)))
    labels_p_train = labels_p.difference(labels_p_test)
    labels_n_train = labels_n.difference(labels_n_test)
    labels_train = list(labels_p_train.union(labels_n_train))
    labels_test = list(labels_p_test.union(labels_n_test))

    random.shuffle(labels_train)
    random.shuffle(labels_test)

    if type(data)==pd.core.frame.DataFrame:

        data_train = data.iloc[labels_train,:]
        data_test = data.iloc[labels_test,:]
        print(data_test.shape)
    else:
        data_train = np.array([data[x] for x in labels_train])
        data_test = np.array([data[x] for x in labels_test])

    if label1 is None:
        return data_train, data_test
    else:
        return data_train, data_test, labels[labels_train], labels[labels_test]

def load_data(dataset):
    # train_pos = "data/group 1/" + dataset + "/train_pos.txt"
    # train_neg = "data/group 1/" + dataset + "/train_neg.txt"
    # test_pos = "data/group 1/" + dataset + "/test_pos.txt"
    # test_neg = "data/group 1/" + dataset + "/test_neg.txt"
    dir1 = dataset.split("_")[0]
    train_pos = "data/" + dir1 + "/" + dataset + "/train_pos.txt"
    train_neg = "data/" + dir1 + "/" + dataset + "/train_neg.txt"
    test_pos = "data/" + dir1 + "/" + dataset + "/test_pos.txt"
    test_neg = "data/" + dir1 + "/" + dataset + "/test_neg.txt"

    data_train_pos = pd.read_table(train_pos, header=None)
    data_train_neg = pd.read_table(train_neg, header=None)
    data_test_pos = pd.read_table(test_pos, header=None)
    data_test_neg = pd.read_table(test_neg, header=None)

    train_seq = data_train_pos.iloc[:, 0].values.tolist() + data_train_neg.iloc[:, 0].values.tolist()
    train_seq_id = [1] * data_train_pos.shape[0] + [0] * data_train_neg.shape[0]
    data_train = pd.DataFrame({"label": train_seq_id, "seq": train_seq})

    # data_train["seq"] = data_train["seq"].apply(lambda x: x.replace("T", 'U'))

    test_seq = data_test_pos.iloc[:, 0].values.tolist() + data_test_neg.iloc[:, 0].values.tolist()
    test_seq_id = [1] * data_test_pos.shape[0] + [0] * data_test_neg.shape[0]
    data_test = pd.DataFrame({"label": test_seq_id, "seq": test_seq})
    # data_test["seq"] = data_test["seq"].apply(lambda x: x.replace("T", 'U'))

    return data_train, data_test


def format_ilearn(seq_df1, is_train):
    seq_df = seq_df1.copy()
    seq_df["is_train"] = [is_train] * seq_df.shape[0]
    seq_id = ["seq_"+str(i) for i in range(seq_df1.shape[0])]
    seq_df["seq_id"] = seq_id
    seq_df = seq_df.loc[:,["seq_id","seq","label","is_train"]]
    return seq_df.values.tolist()

# def repair(encoding):
#     '''
#     Arguments:
#     encoding -- the encoding matrix calculated by iLearns
#     Return:
#     data -- DataFrame，feature matrix with row sample  and column feature
#     label -- list，
#     record_feature_type
#     '''
#
#     encoding = pd.DataFrame(encoding[1:])
#     label = encoding.iloc[:,0].values.astype(int)
#     data = encoding.iloc[:,2:]
#     return data,label

def repair(encoding):
    '''
    Arguments:
    encoding -- the encoding matrix calculated by iLearns
    Return:
    data -- DataFrame，feature matrix with row sample  and column feature
    label -- list，
    record_feature_type
    '''
    encoding = pd.DataFrame(encoding)
    encoding = encoding.iloc[1:,1:]
    label = encoding.iloc[:,0].values.astype(int)
    # data = encoding.iloc[:,2:]
    data = encoding.iloc[:,1:]
    return data,label

def ml_code(seq_df1,is_train="training",feature_extract_method=None):
    '''
    将sequence矩阵转化为特征矩阵
    :param seq_df1: (seq_id,seq)
    :param is_train: training, or testing
    :return:
    '''
    fastas1 = format_ilearn(seq_df1,is_train)
    encodings_trains = []
    label_train = []
    record_feature_type = []
    if feature_extract_method is None:
        # feature_extract_method = {'pse':pse,'ACC':ACC_encoding,"ANF":ANF,"binary":binary,"CKSNAP":CKSNAP,"NCP":NCP,"EIIP":EIIP,"ENAC":ENAC,"Kmer":Kmer,"NAC":NAC,"PseEIIP":PseEIIP}
        feature_extract_method = {"ENAC2": ENAC2, "binary": binary, "NCP": NCP, "EIIP": EIIP}
        print("feature_extract_method:",feature_extract_method.keys())
    for fsm in feature_extract_method.keys():
        method1 = feature_extract_method[fsm]
        d_tr = pd.DataFrame(method1(fastas1))
        data_train,label_train = repair(d_tr)
        encodings_trains.append(data_train)
        record_feature_type.extend([fsm]*data_train.shape[1])
    encoding_TR = pd.concat(encodings_trains,axis=1,ignore_index =True)
    print("encoding_TR:",encoding_TR.shape)
    return encoding_TR.values,np.array(label_train),record_feature_type



def xgb_feature_selection(data_train, label_train):
    data_train2, data_dev,label_train2,label_dev = split_h(data_train, label_train,freq=0.1)
    xgb_model = XGBClassifier(eval_metric="logloss")
    xgb_model.fit(data_train2, label_train2)
    cc = pd.Series(xgb_model.feature_importances_)
    cc = cc.sort_values(ascending = False)
    fs1 = [i for i in cc.index]

    result1 = []
    # for i in range(1,len(fs1),10):
    for i in tqdm(range(1,len(fs1),10), bar_format='{l_bar}%s{bar}%s{r_bar}' % (Fore.RED, Fore.RESET)):
        model = XGBClassifier(eval_metric='logloss')
        # model = LGBMClassifier()
        model.fit(data_train2[:,fs1[:i]], label_train2)
        y_submission_xgb = model.predict_proba(data_dev[:,fs1[:i]])[:, 1]
        y_submission_acc = [(1 if x>0.5 else 0) for x in y_submission_xgb]
        acc1 = accuracy_score(label_dev, y_submission_acc)
        # auc1 = roc_auc_score(label_dev, y_submission_xgb)
        #
        result1.append(acc1)
        # result1.append(auc1)
    index_best = result1.index(max(result1))*10+1
    print("choose",str(index_best),"features")
    return fs1[:index_best]

# def pca(data_train,data_test):
#     scaler = StandardScaler()
#     scaler.fit(data_train)
#     x_train = scaler.transform(data_train)
#     x_test = scaler.transform(data_test)
#     model = PCA(n_components=2).fit(x_train)  # 拟合数据，n_components定义要降的维度
#     x_train1 = model.transform(x_train)  # transform就会执行降维操作
#     x_test1 = model.transform(x_test)  # transform就会执行降维操作
#     return x_train1,x_test1

def pca1(data_train):
    scaler = StandardScaler()
    scaler.fit(data_train)
    x_train = scaler.transform(data_train)
    pca_model = PCA(n_components='mle').fit(x_train)  # 拟合数据，n_components定义要降的维度
    x_train1 = pca_model.transform(x_train)  # transform就会执行降维操作
    return x_train1,scaler,pca_model

# from sklearn.lda import LDA
def lda(data_train,y_train):
    scaler = StandardScaler()
    scaler.fit(data_train)
    x_train = scaler.transform(data_train)
    # model_lda = LinearDiscriminantAnalysis(n_components=2)
    model_lda = LinearDiscriminantAnalysis()
    print("x_train:",x_train.shape)
    model_lda.fit(x_train,y_train)
    X1 = model_lda.transform(x_train)
    return X1,scaler,model_lda

def lso_feature_selection(data_train, label_train):
    data_train2, data_dev,label_train2,label_dev = split_h(data_train, label_train,freq=0.1)
    # xgb_model = XGBClassifier(eval_metric="logloss")
    # xgb_model.fit(data_train2, label_train2)
    # cc = pd.Series(xgb_model.feature_importances_)
    # cc = cc.sort_values(ascending = False)
    # fs1 = [i for i in cc.index]

    # lasso = Lasso(alpha=10 ** (-3))
    lasso = Lasso(alpha=10 ** (-4))
    model_lasso = lasso.fit(data_train, label_train)
    coef = pd.Series(model_lasso.coef_)
    cc = coef[coef != 0].abs().sort_values(ascending=False)
    # cc = coef.abs().sort_values(ascending=False)
    fs1 = [i for i in cc.index]

    result1 = []
    # for i in range(1,len(fs1),10):
    for i in tqdm(range(1,len(fs1),5), bar_format='{l_bar}%s{bar}%s{r_bar}' % (Fore.RED, Fore.RESET)):
        # model = XGBClassifier(eval_metric='logloss')
        model = LGBMClassifier()
        model.fit(data_train2[:,fs1[:i]], label_train2)
        y_submission_xgb = model.predict_proba(data_dev[:,fs1[:i]])[:, 1]
        y_submission_acc = [(1 if x>0.5 else 0) for x in y_submission_xgb]
        acc1 = accuracy_score(label_dev, y_submission_acc)
        # auc1 = roc_auc_score(label_dev, y_submission_xgb)
        #
        result1.append(acc1)
        # result1.append(auc1)
    index_best = result1.index(max(result1))*10+1
    print("choose",str(index_best),"features")
    return fs1[:index_best]


def lasso1(X_train,y_train):
    lasso = Lasso(alpha=10 ** (-3))
    model_lasso = lasso.fit(X_train, y_train)
    coef = pd.Series(model_lasso.coef_, index=X_train.columns)
    print(coef[coef != 0].abs().sort_values(ascending=False))


def remove_corr(data_train,label_train):
    data_train = data_train.astype("float")
    data_train.columns = range(data_train.shape[1])
    data_corr = abs(data_train.corr())
    data_corr.index = range(data_train.shape[1])
    data_corr.columns = range(data_train.shape[1])
    corr_label = abs(data_train.corrwith(pd.Series(label_train)))
    corr_label.index = range(data_train.shape[1])
    # 待删除
    col_remain = data_train.columns.copy()
    print("col_remain:",col_remain)
    print("data_corr.index:",data_corr.index)
    for col1 in col_remain:
        if col1 in data_corr.index:
            data_row_to_delete = (data_corr.loc[col_remain[col1], :]) > 0.8  # 第0行中大于0.8的列，只保留与label最相关的
            # print("data_row_to_delete:",data_row_to_delete)
            to_delete = corr_label[data_row_to_delete]
            print("to_delete1:", to_delete,len(to_delete))
            choose_remain = to_delete[to_delete == to_delete.max()]  # 计算上面的待删除中，与label最相关的
            print("choose_remain:", choose_remain.index)
            to_delete = to_delete.drop(choose_remain.index)
            to_delete = to_delete.index
            print("to_delete2:", to_delete)
            data_corr = data_corr.drop(to_delete, axis=1)
            data_corr = data_corr.drop(to_delete, axis=0)
            print("select data_corr:", data_corr.shape)
        else:
            continue
    print("select data_corr last:",data_corr.shape)
    return data_corr.index
